/*
 *
 * Copyright (c) 2000 Scott Oaks and Henry Wong. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted.
 *
 * This sample source code is provided for example only,
 * on an unsupported, as-is basis. 
 *
 * AUTHOR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  AUTHOR
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */


import java.rmi.*;
import net.jini.discovery.*;
import net.jini.core.lease.*;
import net.jini.core.event.*;
import net.jini.core.transaction.*;
import net.jini.core.transaction.server.*;
import net.jini.admin.*;
import net.jini.lease.*;

public class ConvertClient {

    public static void main(String[] args) throws Exception {
        System.setSecurityManager(new RMISecurityManager());

        ServiceFinder sf = new ServiceFinder(TransactionManager.class);
        TransactionManager txm = (TransactionManager)(sf.getObject());

        Transaction.Created resp =
                    TransactionFactory.create(txm, Lease.FOREVER);
        LeaseRenewalManager lrm = new LeaseRenewalManager();
        lrm.renewUntil(resp.lease, Lease.FOREVER, null);

        boolean commit = true;
        try {
            // Do some transaction based operations here, passing
            // resp.transaction to services that need it
        } catch (Exception e) {
            commit = false;
        }

        if (commit)
            resp.transaction.commit();
        else resp.transaction.abort();

        // The transaction lease was cancelled as a side-effect of
        // the commit
        lrm.remove(resp.lease);
    }
}
